﻿using Hotline.Article;
using Hotline.FlowEngine.Workflows;
using Hotline.Orders;
using Hotline.Share.Dtos.Article;
using Hotline.Share.Dtos.Order;
using Hotline.Share.Enums.Article;
using Hotline.Users;
using Microsoft.AspNetCore.Mvc;
using XF.Domain.Authentications;
using XF.Domain.Exceptions;
using XF.Domain.Repository;

namespace Hotline.Api.Controllers.OrderApi;

public class OrderComplementController : BaseController
{
    private readonly IOrderDomainService _orderDomainService;
    private readonly ISessionContext _sessionContext;
    private readonly IRepository<OrderComplement> _orderComplementRepository;
    private readonly IRepository<WorkflowStep> _workflowStepRepository;
    private readonly IRepository<User> _userRepository;
    private readonly ICircularRecordDomainService _circularRecordDomainService;

    public OrderComplementController(IOrderDomainService orderDomainService,
       ISessionContext sessionContext,
       IRepository<OrderComplement> orderComplementRepository,
       IRepository<WorkflowStep> workflowStepRepository,
       IRepository<User> userRepository,
       ICircularRecordDomainService circularRecordDomainService)
    {
        _orderDomainService = orderDomainService;
        _sessionContext = sessionContext;
        _orderComplementRepository = orderComplementRepository;
        _workflowStepRepository = workflowStepRepository;
        _userRepository = userRepository;
        _circularRecordDomainService = circularRecordDomainService;
    }
    
    /// <summary>
    /// 补充
    /// </summary>
    /// <param name="dto"></param>
    /// <returns></returns>
    [HttpPost]
    public async Task<string> Supplement([FromBody] AddOrderComplementDto dto)
    {
        return await _orderDomainService.AddOrderComplementAsync(dto, HttpContext.RequestAborted);
    }

    #region 添加补充

    /// <summary>
    /// 添加补充 _notificationWaitSendRepository
    /// </summary>
    /// <param name="dto"></param>
    /// <returns></returns>
    [HttpPost("add_order_complement")]
    public async Task AddOrderComplement([FromBody] AddComplementDto dto)
    {
        if (dto == null)
            throw UserFriendlyException.SameMessage("数据错误！");
        if (string.IsNullOrEmpty(dto.Opinion))
            throw UserFriendlyException.SameMessage("补充内容不能为空！");
        if (dto.Opinion.Length > 2000)
            throw UserFriendlyException.SameMessage("补充内容限制2000字！");
        var data = await _orderDomainService.GetOrderAsync(dto.OrderId, cancellationToken: HttpContext.RequestAborted);
        if (data == null)
            throw UserFriendlyException.SameMessage("工单查询失败！");

        OrderComplement complement = new OrderComplement()
        {
            OrderId = dto.OrderId,
            Opinion = dto.Opinion,
            SupplyName = _sessionContext.UserName,
            SupplyTime = DateTime.Now,
            No = data.No,
            IsProComplement = false
        };

        var id = await _orderComplementRepository.AddAsync(complement, HttpContext.RequestAborted);
        if (!string.IsNullOrEmpty(id))
        {
            #region 处理推送消息

            //获取当前办理节点数据
            var work = await _workflowStepRepository.GetAsync(p => p.Id == data.ActualHandleStepId, HttpContext.RequestAborted);
            if (work != null)
            {
                //获取办理指定类型
                var workflowStepHandler = work.GetWorkflowStepHandler();
                if (workflowStepHandler != null)
                {
                    AddCircularDto circularDto = new AddCircularDto()
                    {
                        Title = "工单补充",
                        Content = "工单" + data.No + "有补充内容，请注意查收！",
                        CircularTypeId = "6",
                        CircularTypeName = "工单补充",
                        IsMustRead = true,
                        SourceOrgId = _sessionContext.RequiredOrgId,
                        SourceOrgName = _sessionContext.OrgName,
                        CircularType = ECircularType.Person
                    };

                    List<CircularReadGroupDto> users = [];
                    if (!string.IsNullOrEmpty(workflowStepHandler.UserId)) //指定用户
                    {
                        users.Add(new CircularReadGroupDto()
                        {
                            UserId = workflowStepHandler.UserId,
                            UserName = workflowStepHandler.Username
                        });
                    }
                    else if (!string.IsNullOrEmpty(workflowStepHandler.RoleId)) //指定角色
                    {
                        //查询指定角色下面所有的用户
                        var userlist = await _userRepository.Queryable().Where(x =>
                                x.OrgId == workflowStepHandler.OrgId && x.Roles.Any(d => workflowStepHandler.RoleId.Contains(d.Id)))
                            .Select(d => new CircularReadGroupDto
                            {
                                UserId = d.Id,
                                UserName = d.Name
                            }).ToListAsync();
                        users.AddRange(userlist);
                    }
                    else if (!string.IsNullOrEmpty(workflowStepHandler.OrgId)) //指定部门
                    {
                        users.Add(new CircularReadGroupDto()
                        {
                            OrgId = workflowStepHandler.OrgId,
                            OrgName = workflowStepHandler.OrgName
                        });
                        circularDto.CircularType = ECircularType.Org;
                    }

                    if (users != null && users.Count > 0)
                    {
                        circularDto.CircularReadGroups = users;
                        //调用推送消息通用接口
                        await _circularRecordDomainService.AddCircularMessage(circularDto, HttpContext.RequestAborted);
                    }
                }
            }

            #endregion
        }
    }

    #endregion
}